package zero1hd.rhythmbullet.audio.analyzer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.utils.FloatArray;

import zero1hd.rhythmbullet.audio.AudioDataPackage;
import zero1hd.rhythmbullet.audio.MusicManager;

public class PeakDetectionRunnable implements Runnable {
	boolean work = true, done;
	private FloatArray bassPrunned;
	private FloatArray mPrunned;
	private FloatArray umPrunned;
	private FloatArray bassPeaks = new FloatArray();
	private FloatArray mPeaks = new FloatArray();
	private FloatArray umPeaks = new FloatArray();
	private float bassMaxValue;
	private float mMaxValue;
	private float umMaxValue;
	private float secondsPerWindow;
	private float mAvg;
	private float bassAvg;
	private float umAvg;
	private MusicManager musicManager;
	private AudioDataPackage pack;
	private int PUID;
	
	public PeakDetectionRunnable(PruneFluxRunnable pfr, int PUID) {
		bassPrunned = pfr.getBassPrunned();
		mPrunned = pfr.getmPrunned();
		umPrunned = pfr.getUmPrunned();
		musicManager = pfr.getMusicManager();
		this.PUID = PUID;
	}
	
	@Override
	public void run() {
		int lastBeatID = 0;
		float bassBeats = 0;
		float mBeats = 0;
		float umBeats = 0;
		float avgSPB = -1f;

		for (int i = 0; i < umPrunned.size-1 && work; i++) {
			bassPeaks.add((bassPrunned.get(i) > bassPrunned.get(i+1) ? bassPrunned.get(i) : 0f));
			if (bassPeaks.get(i) > bassMaxValue) {
				bassMaxValue = bassPeaks.get(i);
			}
			
			mPeaks.add((mPrunned.get(i) > mPrunned.get(i+1) ? mPrunned.get(i) : 0f));
			if (mPeaks.get(i) > mMaxValue) {
				mMaxValue = mPeaks.get(i);
			}
			
			umPeaks.add((umPrunned.get(i) > umPrunned.get(i+1) ? umPrunned.get(i) : 0f));
			if (umPeaks.get(i) > umMaxValue) {
				umMaxValue = umPeaks.get(i);
			}
			

			if (avgSPB != -1) {
				if (bassPeaks.get(i) == 0) {
					avgSPB ++;
				} else {
					lastBeatID = i;
				}
			} else if (bassPeaks.get(i) != 0) {
				avgSPB = 0;
			}

			if (bassPeaks.get(i) != 0) {
				bassAvg += bassPeaks.get(i);
				bassBeats++;
			}
			
			if (mPeaks.get(i) != 0) {
				mAvg += mPeaks.get(i);
				mBeats++;
			}
			
			if (umPeaks.get(i) != 0) {
				umAvg += umPeaks.get(i);
				umBeats++;
			}
		}
		
		secondsPerWindow = musicManager.getReadWindowSize()/musicManager.getSampleRate();

		//then we minus one from the beats so it actually works out
		avgSPB -= bassPrunned.size-lastBeatID;
		avgSPB *= secondsPerWindow;
		avgSPB /= bassBeats;
		Gdx.app.debug("Audio Analyzer", "Avg SPB: " + avgSPB);

		bassAvg /= bassBeats;
		mAvg /= mBeats;
		umAvg /= umBeats;
		Gdx.app.debug("Audio Analyzer", "Avg bass: " + bassAvg);
		Gdx.app.debug("Audio Analyzer", "Avg M: " + mAvg);
		Gdx.app.debug("Audio Analyzer", "Avg UM: " + umAvg);
		
		pack = new AudioDataPackage();
		pack.setBassData(bassPeaks, bassMaxValue, bassAvg);
		pack.setmData(mPeaks, mMaxValue, mAvg);
		pack.setUmData(umPeaks, umMaxValue, umAvg);
		
		pack.setPUID(PUID);
		pack.setAvgSPB(avgSPB);
		pack.setSecPerWin(secondsPerWindow);
		pack.setMusicInfo(musicManager);
		if (work) {
			Gdx.app.debug("Audio Analyzer", "data cleaned and ready for map gen.");
		}
	}
	
	public AudioDataPackage getPack() {
		return pack;
	}
	
	public boolean isDone() {
		return done;
	}
}
