package zero1hd.rhythmbullet.graphics.ui.components;

import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.VerticalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.WidgetGroup;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.rhythmbullet.audio.SongInfo;
import zero1hd.rhythmbullet.graphics.ui.pages.MusicSelectionPage;

public class MusicSelectable extends WidgetGroup implements Disposable {
	private Table table;

	private Image imageIcon;
	private ShortenedTextLabel displayName;
	private Label durationLabel;
	private ShortenedTextLabel authorLabel;

	private FileHandle musicFile;

	private Texture albumCover;
	private SongInfo songInfo;
	
	private boolean selected;
	
	private VerticalGroup vGroup;
	
	private MusicSelectionPage msp;
	
	public MusicSelectable(FileHandle musicFile, Preferences musicAnnotation, Skin skin, Texture defaultAlbumC, MusicSelectionPage msp) {
		table = new Table(skin);
		table.setBackground("holo-pane");
		table.setFillParent(true);
		vGroup = new VerticalGroup();
		this.msp = msp;
		setName(musicFile.name());
		table.defaults().pad(2f);

		this.albumCover = defaultAlbumC;
		this.musicFile = musicFile;
		songInfo = new SongInfo(musicFile, musicAnnotation);

		imageIcon = new Image(albumCover);
		table.add(imageIcon).size(180f).left().expandX();

		displayName = new ShortenedTextLabel(musicFile.nameWithoutExtension().replace('_', ' '), skin, "sub-font", skin.getColor("default"));
		vGroup.addActor(displayName);

		durationLabel = new Label("Loading...", skin, "sub-font", skin.getColor("default"));
		vGroup.addActor(durationLabel);

		authorLabel = new ShortenedTextLabel("Loading...", skin, "sub-font", skin.getColor("default"));
		vGroup.addActor(authorLabel);

		table.add(vGroup).expandX().fillX().center();
		
		addActor(table);
		addListener(new ClickListener() {
			@Override
			public void clicked(InputEvent event, float x, float y) {
				select();
				super.clicked(event, x, y);
			}
		});
	}
	
	/**
	 * updates the UI side of information.
	 * needs to be called in thread with gl context.
	 */
	public void updateInfo() {
		displayName.setOriginalText(songInfo.getMusicName());
		durationLabel.setText("Runtime: "
				+ ((songInfo.getDurationInSeconds() / 60 < 1) ? "00" : songInfo.getDurationInSeconds() / 60) + ":"
				+ ((songInfo.getDurationInSeconds() - (songInfo.getDurationInSeconds() / 60) * 60) < 10
						? "0" + (songInfo.getDurationInSeconds() - (songInfo.getDurationInSeconds() / 60) * 60)
						: (songInfo.getDurationInSeconds() - (songInfo.getDurationInSeconds() / 60) * 60)));
		authorLabel.setOriginalText("Author: " + songInfo.getAuthor());
		authorLabel.setToOriginalText();
		songInfo.setupTexture(albumCover);
		albumCover = songInfo.getAlbumCover();
		imageIcon.setDrawable((new TextureRegionDrawable(new TextureRegion(albumCover))));
	}
	
	
	@Override
	public void layout() {
		displayName.setTargetWidth((int) (getWidth() - 300));
		authorLabel.setTargetWidth((int) (getWidth() - 300));
		displayName.resize();
		super.layout();
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	public FileHandle getMusicFile() {
		return musicFile;
	}

	public SongInfo getAudioInfo() {
		return songInfo;
	}

	@Override
	public void dispose() {
		songInfo.dispose();
	}

	public boolean isMusicInvalid() {
		return songInfo.isInvalidMusic();
	}
	
	/**
	 * Selects this panel
	 * @param updateGlobalSelection whether or not to tell everything that this is selected. Usually true unless you set the global somewhere else.
	 */
	public void select() {
		msp.deselectAll();
		table.setBackground("holo-pane-down");
		selected = true;
		msp.setCurrentlySelected(this);
	}
	
	public void deselect() {
		table.setBackground("holo-pane");
		selected = false;
	}
	
	public boolean isSelected() {
		return selected;
	}
	
	@Override
	public float getPrefHeight() {
		return table.getMinHeight();
	}
}
